# email2toppy.py - setup your Topfield 5100PVR timers using email.
# See http://www.iki.fi/ike/email2toppy
#
# version 0.1 - 2006-06-09 - by Ilkka Salminen (ike@iki.fi)
# This Python script is released to public domain. You are allowed to
# use it as you like. Comments, bugs, etc. are welcomed. 
import sys, imaplib, socket, re, ConfigParser, logging, smtplib, os

DEF_INIFILE = 'email2toppy.cfg'

def readConfig(inifile):
	"""Reads the inifile. 
	Returns settings configuration object"""
	
	config = ConfigParser.SafeConfigParser()
	f = config.read(inifile)
	if(len(f) == 0): 
		return None
	else:
		return config

def setLogParameters(params):
	try:
		loglevel = int(params['loglevel'])
	except KeyError:
		loglevel = 50
	try:
		logfile = params['logfile']
	except KeyError:
		logfile = DEF_INIFILE
	logging.basicConfig(level=loglevel,
			format='%(asctime)s %(levelname)s %(message)s',
			filename=logfile,
            filemode='a')

def getImapMail(imapSettings):
	"""Gets messages from IMAP server. 
	Returns new messages with given tag in subject."""
	try:
		tag = imapSettings["tag"]
		mailbox = imaplib.IMAP4(imapSettings["server"])
		mailbox.login(imapSettings["user"], imapSettings["password"])
	except KeyError, error:
		logging.critical("Missing IMAP configuration: "+str(error))
		return None
	except imaplib.IMAP4.error, error:
		logging.error("Cannot connect to IMAP server: " + str(error))
		return None
	except socket.error, error:
		logging.error("Network error while connecting to IMAP server: "+str(error))
		return None
		
	imapSearchString = '(UNSEEN SUBJECT "'+tag+'")'
	logging.debug("Using IMAP search string: "+imapSearchString)
	messageList = []
	try:
		mailbox.select()
		ok, data = mailbox.search(None, imapSearchString)
		if(ok != 'OK'): 
			logging.error("IMAP server error: "+ok)
			return
		else: 
			logging.info("Connected to IMAP server")
		msgNums = data[0].split();
		logging.debug("Fetching messages: "+ str(msgNums))
		for msg in msgNums:
			ok, data = mailbox.fetch(msg, '(FLAGS RFC822)')
			messageList.append(data[0][1])
	except imaplib.IMAP4.error, error:
		logging.error("IMAP error: "+str(error))
	
	logging.info("Closing IMAP connection")
	mailbox.close()
	mailbox.logout()
	return messageList		

def makeCommandFile(cmdFileName, realPin, emails):
	"""Creates commandfile to be sent to Toppy. 
	realPin must match pin in email. emails is list of emails containing timer data.
	Returns None if no valid PIN found"""
	logging.debug("Creating command data")
	if(len(cmdFileName) == 0 or len(emails) == 0):
		logging.error("makeCommandFile: nothing to do")
		return False
	keys = ('PIN', 'channel', 'command', 'date', 'start', 'duration', 'type', 'repeat', 'name')
	regExpKeys = '|'.join(keys)
	timerItemRegExp = r'('+regExpKeys+r')=(.+)\r\n'
	rexp = re.compile(timerItemRegExp)
	command = ""
	validPin = False
	for msg in emails:	
		items = dict(rexp.findall(msg))
		if('PIN' in items and items['PIN'] == realPin):
			validPin = True
		try:
			command += "Command."+items["command"]+":"+items["channel"]+","+items["date"]+","+items["start"]+","+items["duration"]+","+"TV"+","+items["repeat"]+","+items["type"]+";"+items["name"]
			command += "\n"
		except KeyError, error:
			logging.info("Incomplete command, missing parameter "+str(error))

	if(not validPin):
		logging.info("Got invalid PIN")	
		return False
	else:
		command += "Command.CreateTimerList\n"
		logging.info("Got JAG command: "+command)
		try:
			f = file(cmdFileName, 'w')
			f.write(command)
			f.close()
		except IOError, error:
			logging.critical("Error creating commandfile: "+str(error))
			return False
	return True
			
def sendFileToToppy(transferSettings):
	"""Sends JAG's EPG remote command to Toppy."""
	logging.debug("Sending commands to Toppy")
	try:
		command = transferSettings['writecmd']
	except KeyError, error:
		logging.critical("Missing transfer configuration: "+str(error))
		return False
	logging.info("Issuing command "+command)
	retVal = os.system(command)
	if(retVal != 0):
		logging.error("Sending commandfile failed")
		return False
	
	return True
	
	
def getResultsFromToppy(transferSettings):
	"""Reads timerlist from Toppy
	Return string containing the timerlist"""
	logging.debug("Reading timerlist from Toppy")
	try:
		readCommand = transferSettings['readcmd']
		deleteCommand = transferSettings['deletecmd']
	except KeyError, error:
		logging.critical("Missing transfer configuration: "+str(error))
		return False
		
	logging.info("Issuing command "+readCommand)
	retVal = os.system(readCommand)
	if(retVal != 0):
		logging.info("Reading results failed. Maybe no results or no connection to Toppy")
		return False
	elif(deleteCommand.strip() != ''):
		logging.debug("Issuing command: "+deleteCommand)
		retVal = os.system(deleteCommand)
		if(retVal != 0):
			logging.error("Could not delete returnfile")
			return False
	
	return True

	
def emailData(fileName, smtpSettings):
	"""Emails data to address"""
	try:
		fromAddr = smtpSettings['from']
		toAddr = smtpSettings['replyto']
		logging.info("Emailing results to "+toAddr)
		headers = "From: "+fromAddr
		headers += "\r\n" + "To: "+toAddr
		headers += "\r\nSubject: "+smtpSettings['subject']
		server = smtpSettings['server']
	except KeyError, error:
		logging.critical("Missing SMTP configuration: "+str(error))
		return
	try:
		f = open(fileName)
		body = f.read()
		f.close()
	except IOError, error:
		logging.info("Can't read resultfile: "+str(error))
		return
		
	msg = headers+"\r\n\r\n"+body+"\r\n"
	try:
		server = smtplib.SMTP(server)
		server.sendmail(fromAddr, toAddr, msg)
		server.quit()
		logging.info("Email sent ok")
	except SMTPException, error:
		logging.critical("Error sending reply: "+str(error))


	
if __name__ == "__main__":
	if(len(sys.argv) > 1):
		inifile = sys.argv[1]
	else:
		inifile = DEF_INIFILE
	config = readConfig(inifile)
	if(config == None):
		logging.critical("No config file")
		sys.exit(1)
	setLogParameters(dict(config.items('log')))
	logging.info("Configuration file used: "+inifile)

	imapSettings = dict(config.items('imap'))
	mails = []
	commandData = None
	realPin = config.get('general', 'pin')
	if(realPin == ''):
		logging.info("No PIN defined - no emails")
	else:
		if(len(imapSettings) == 0):
			logging.info("No IMAP settings defined - can only return data from Toppy")
		else:
			mails = getImapMail(imapSettings)
	ok = False
	transferSettings = dict(config.items('transfer'))
	if(mails != None and len(mails) > 0):
		ok = False
		try:
			ok = makeCommandFile(transferSettings['localdir']+transferSettings['commandfile'], realPin, mails)
		except KeyError, error:
			logging.error("Undefined configuration setting: "+str(error))
		if(ok):
			sendFileToToppy(transferSettings)
	else:
		logging.info("No messages")
		
	ok = getResultsFromToppy(transferSettings)
	if(ok):
		try:
			emailData(transferSettings['localdir']+transferSettings['returnfile'], dict(config.items('smtp')))
		except KeyError, error:
			logging.error("Missing configuration parameter: "+str(error))
		
	